@@ -25,7 +25,6 @@ import argparse
 
 # to find the magic library
 import magic
-
 import subprocess
 
 def id_generator(size=10, chars=string.ascii_uppercase + string.digits):
@@ -39,7 +38,7 @@ def unpack(input):
         file_type = m.from_file(input)
     except Exception as e:
         raise dxpy.AppError("Error while identifying compression format: " + str(e))
-
+    
     # if we find a tar file throw a program error telling the user to unpack it
     if file_type == 'application/x-tar':
         raise dxpy.AppError("App does not support tar files.  Please unpack.")
@@ -58,7 +57,7 @@ def unpack(input):
         # just return input filename since it's already uncompressed
         return input
 
-    if uncomp_util != None:
+    if uncomp_util != None:        
         # bzcat does not support -t.  Use non streaming decompressors for testing input
         test_util = None
         if uncomp_util == 'xzcat':
@@ -85,7 +84,7 @@ def unpack(input):
         raise dxpy.AppError("Found a tar archive after decompression.  Please untar your files before importing")
     elif 'ASCII text' not in uncomp_type:
         raise dxpy.AppError("After decompression found file type other than plain text")
-
+    
     try:
         out_name = id_generator()
         subprocess.check_call(" ".join([uncomp_util, "--stdout", input, ">", out_name]), shell=True)
@@ -93,19 +92,27 @@ def unpack(input):
     except subprocess.CalledProcessError as e:
         raise dxpy.AppError("Unable to open compressed input for reading: " + str(e))
 
-
 def detect_type(bed_file):
-    num_cols = find_num_columns(bed_file)
-    if num_cols == 12:
-        bed_type = "genes"
-    elif num_cols > 3:
-        bed_type = "named_spans"
+    delimiter = find_delimiter(bed_file)
+    with open(bed_file, 'rU') as bf:
+        header=""
+        while "track" not in header:
+            header=bf.readline()
+            # if this isn't a browser line either then there isn't a header
+            if "browser" not in header:
+                break
+        if "type=bedDetail" in header:
+            print >> sys.stderr, "File is a BED detail file"
+            return {"type": "bedDetail", "delimiter": delimiter}
+            
+    num_cols = find_num_columns(bed_file, delimiter)
+    if num_cols >= 12:
+        return {"type": "genes", "delimiter": delimiter}
     else:
-        bed_type = "spans"
-
-    return bed_type
-
+        return {"type": "spans", "delimiter": delimiter}
+    
 # takes the whole bed file and splits into separate files for each track contained in it
+
 def split_on_track(bed_file):
     files = []
     current_filename = id_generator()
@@ -129,13 +136,13 @@ def split_on_track(bed_file):
                 curr_file.close()
                 files.append(current_filename)
                 break
-
+            
             curr_file.write(line)
             line = bf.readline()
 
     return files
 
-def find_num_columns(bed_file):
+def find_num_columns(bed_file, delimiter="\t"):
     num_cols = 0
 
     with open(bed_file, "rU") as bf:
@@ -143,64 +150,38 @@ def find_num_columns(bed_file):
         while line != "":
             if line.startswith("track"):
                 line = bf.readline()
-            line = line.split()
+            line = line.split(delimiter)
             if len(line) > num_cols:
                 num_cols = len(line)
             line = bf.readline()
 
     print >> sys.stderr, "Found num cols: " + str(num_cols)
-
     return num_cols
 
-def import_spans(bed_file, table_name, ref_id, file_id, additional_types, property_keys, property_values, tags):
-    num_cols = find_num_columns(bed_file)
-    if num_cols < 3:
-        raise dxpy.AppError("BED file contains less than the minimum 3 columns.  Invalid BED file.")
-
-    columns = [("chr", "string"),
-               ("lo", "int32"),
-               ("hi", "int32")]
-
-    column_descs = [dxpy.DXGTable.make_column_desc(name, type) for name, type in columns]
-    gri_index = dxpy.DXGTable.genomic_range_index("chr", "lo", "hi")
-
-    with open(bed_file, 'rU') as bed, dxpy.new_dxgtable(column_descs, indices=[gri_index], mode='w') as span:
-        span_table_id = span.get_id()
-
-        details = {"original_contigset": dxpy.dxlink(ref_id)}
-        if file_id != None:
-            details["original_file"] = dxpy.dxlink(file_id)
-        if len(property_keys) != len(property_values):
-            raise dxpy.AppError("Expected each provided property to have a corresponding value.")
-        for i in range(len(property_keys)):
-            details[property_keys[i]] = property_values[i]
-        span.set_details(details)
-
-        span.add_types(["Spans","gri"])
-        span.rename(table_name)
-
-        for line in bed:
-            if line.startswith("track"):
-                details = span.get_details()
-                details['track'] = line
-                span.set_details(details)
-                continue
-            line = line.rstrip("\n")
-            line = line.split()
-            if len(line) == 0:
-                break
-            if len(line) < 3:
-                raise dxpy.AppError("Line: "+"\t".join(line)+" in BED file contains less than the minimum 3 columns.  Invalid BED file.")
-            line[1] = int(line[1])
-            line[2] = int(line[2])
-
-            span.add_row(line)
-
-    return dxpy.dxlink(span.get_id())
-
-def import_named_spans(bed_file, table_name, ref_id, file_id, additional_types, property_keys, property_values, tags):
-    num_cols = find_num_columns(bed_file)
+def find_delimiter(bed_file):
+    with open(bed_file, "rU") as bf: 
+        line = bf.readline()
+        if line.startswith("track"):
+            line = bf.readline()
+        tab_split = line.split("\t")
+        
+        if len(tab_split) >= 3: 
+            print >> sys.stderr, "Bed file is tab delimited"
+            return "\t"
+        else: 
+            space_split = line.split()
+            if len(space_split) < 3: 
+                raise dxpy.AppError("File is not a valid bed file (neither space delimited nor tab delimited")
+            print >> sys.stderr, "Bed file is space delimited"
+            return " "
+            
+def import_spans(bed_file, table_name, ref_id, file_id, additional_types, property_keys, property_values, tags, isBedDetail, delimiter="\t"):
+    num_cols = find_num_columns(bed_file, delimiter)
 
+    # if this is a bedDetail file we should treat the last two columns separately
+    if isBedDetail:
+        num_cols -= 2
+    
     possible_columns = [("chr", "string"),
                         ("lo", "int32"),
                         ("hi", "int32"),
@@ -211,10 +192,16 @@ def import_named_spans(bed_file, table_name, ref_id, file_id, additional_types,
                         ("thick_end", "int32"),
                         ("item_rgb", "string")]
 
+    bedDetail_columns = [("bedDetail_ID", "string"),
+                         ("bedDetail_desc", "string")]
+
     possible_default_row = ["", 0, 0, "", 0, ".", 0, 0, ""]
 
     columns = possible_columns[:num_cols]
 
+    if isBedDetail:
+        columns.extend(bedDetail_columns)
+
     if num_cols > len(columns):
         for i in range(len(columns), num_cols):
             columns.append(("BED_column_"+str(i+1), "string"))
@@ -222,15 +209,21 @@ def import_named_spans(bed_file, table_name, ref_id, file_id, additional_types,
 
     default_row = possible_default_row[:num_cols]
 
+    if isBedDetail:
+        default_row.extend(["",""])
+
     column_descs = [dxpy.DXGTable.make_column_desc(name, type) for name, type in columns]
-
-    indices = [dxpy.DXGTable.genomic_range_index("chr","lo","hi", 'gri'),
-               dxpy.DXGTable.lexicographic_index([
-                  dxpy.DXGTable.lexicographic_index_column("name", True, False),
-                  dxpy.DXGTable.lexicographic_index_column("chr"),
-                  dxpy.DXGTable.lexicographic_index_column("lo"),
-                  dxpy.DXGTable.lexicographic_index_column("hi")], "search")]
-
+    
+    indices = [dxpy.DXGTable.genomic_range_index("chr","lo","hi", 'gri')]
+    for c in columns:
+        if "name" in c:
+            indices.append(dxpy.DXGTable.lexicographic_index([
+                              dxpy.DXGTable.lexicographic_index_column("name", True, False),
+                              dxpy.DXGTable.lexicographic_index_column("chr"),
+                              dxpy.DXGTable.lexicographic_index_column("lo"),
+                              dxpy.DXGTable.lexicographic_index_column("hi")], "search"))
+            break
+            
     with open(bed_file, 'rU') as bed, dxpy.new_dxgtable(column_descs, indices=indices, mode='w') as span:
         details = {"original_contigset": dxpy.dxlink(ref_id)}
         if file_id != None:
@@ -238,8 +231,8 @@ def import_named_spans(bed_file, table_name, ref_id, file_id, additional_types,
         if len(property_keys) != len(property_values):
             raise dxpy.AppError("Expected each provided property to have a corresponding value.")
         for i in range(len(property_keys)):
-            details[property_keys[i]] = property_values[i]
-
+            details[property_keys[i]] = property_values[i]    
+    
         span.set_details(details)
 
         span.add_types(["Spans", "gri"])
@@ -254,7 +247,16 @@ def import_named_spans(bed_file, table_name, ref_id, file_id, additional_types,
                 span.set_details(details)
                 continue
             line = line.rstrip("\n")
-            line = line.split()
+            line = line.split(delimiter)
+            if isBedDetail:
+                # only the first 4 columns are guaranteed to be defined by UCSC
+                validate_line(line[:4])
+                # save last two fields separately
+                bedDetailFields = line[-2:]
+                line = line[:-2]     
+            else:        
+                validate_line(line[:num_cols])
+            
             # check to see if this is a weird line
             if len(line) == 0:
                 break
@@ -267,16 +269,16 @@ def import_named_spans(bed_file, table_name, ref_id, file_id, additional_types,
                 row[2] = int(line[2])
                 row[3] = line[3]
                 # dashes are sometimes used when field is invalid
-                if line[4] == "-":
+                if line[4] == "-" or line[4] == ".":
                     line[4] = 0
-                row[4] = int(line[4])
+                row[4] = float(line[4])
                 row[5] = line[5]
                 # dashes are sometimes used when field is invalid
-                if line[6] == "-":
+                if line[6] == "-" or line[6] == ".":
                     line[6] = 0
                 row[6] = int(line[6])
                 # dashes are sometimes used when field is invalid
-                if line[7] == "-":
+                if line[7] == "-" or line[7] == ".":
                     line[7] = 0
                 row[7] = int(line[7])
                 row[8] = line[8]
@@ -287,9 +289,16 @@ def import_named_spans(bed_file, table_name, ref_id, file_id, additional_types,
             # value error when fields are messed up and string gets converted to int, etc.  Throw these out.
             except ValueError:
                 continue
-
+            
+            if isBedDetail:
+                # add these in at the end if we have a bedDetail file
+                row[num_cols] = bedDetailFields[0]
+                row[num_cols+1] = bedDetailFields[1]
+            
             span.add_row(row)
 
+        span.flush()
+
     return dxpy.dxlink(span.get_id())
 
 
@@ -313,7 +322,7 @@ def generate_gene_row(line, block_size, block_start, span_type, default_row, par
         if parent_id != -1:
             row[2] = row[1] + block_size
         else:
-            row[2] = int(line[2])
+            row[2] = int(line[2])        
 
         # name
         row[3] = line[3]
@@ -342,7 +351,7 @@ def generate_gene_row(line, block_size, block_start, span_type, default_row, par
         row[9] = -1
 
         # description
-        row[10] = ""
+        row[10] = "\t".join(line[12:])
         # BED files have no description?
 
     # a misformed line can have string columns where they should be int
@@ -355,7 +364,7 @@ def generate_gene_row(line, block_size, block_start, span_type, default_row, par
     return row
 
 
-def import_genes(bed_file, table_name, ref_id, file_id, additional_types, property_keys, property_values, tags):
+def import_genes(bed_file, table_name, ref_id, file_id, additional_types, property_keys, property_values, tags, delimiter="\t"):
     # implement BED importing from this format:
     # http://genome.ucsc.edu/FAQ/FAQformat.html#format1
 
@@ -372,8 +381,8 @@ def import_genes(bed_file, table_name, ref_id, file_id, additional_types, proper
                ("description", "string")]
 
     column_descs = [dxpy.DXGTable.make_column_desc(name, type) for name, type in columns]
-
-    indices = [dxpy.DXGTable.genomic_range_index("chr","lo","hi", 'gri'),
+    
+    indices = [dxpy.DXGTable.genomic_range_index("chr","lo","hi", 'gri'), 
                dxpy.DXGTable.lexicographic_index([
                   dxpy.DXGTable.lexicographic_index_column("name", True, False),
                   dxpy.DXGTable.lexicographic_index_column("chr"),
@@ -409,7 +418,8 @@ def import_genes(bed_file, table_name, ref_id, file_id, additional_types, proper
                 continue
             line = line.rstrip("\n")
             row = list(default_row)
-            line = line.split()
+            line = line.split(delimiter)
+            validate_line(line)
             if len(line) < 12:
                 raise dxpy.AppError("Line: "+"\t".join(line)+" in gene model-like BED file contains less than 12 columns.  Invalid BED file.")
 
@@ -418,8 +428,8 @@ def import_genes(bed_file, table_name, ref_id, file_id, additional_types, proper
             if row != None:
                 span.add_row(row)
                 current_parent_id = current_span_id
-                current_span_id += 1
-
+                current_span_id += 1          
+                
                 # add all children
                 blockCount = int(line[9])
                 line[10] = line[10].rstrip(",").split(",")
@@ -433,18 +443,18 @@ def import_genes(bed_file, table_name, ref_id, file_id, additional_types, proper
                 # set thick* to be within the gene if outside
                 thickStart = min(max(int(line[6]), gene_lo), gene_hi)
                 thickEnd = max(min(int(line[7]), gene_hi), gene_lo)
-
+                
                 for i in range(blockCount):
                     # look to thickStart and thickEnd to get information about the type of this region
                     # if thick* are the same or cover the whole transcript then we ignore them
                     # else, we partition the exons into CDS and UTR based on their boundaries
                     if thickStart == thickEnd or (thickStart == gene_lo and thickEnd == gene_hi):
-                        span.add_row(generate_gene_row(line,
-                                                       blockSizes[i],
-                                                       blockStarts[i],
-                                                       "exon",
-                                                       default_row,
-                                                       current_parent_id,
+                        span.add_row(generate_gene_row(line, 
+                                                       blockSizes[i], 
+                                                       blockStarts[i], 
+                                                       "exon", 
+                                                       default_row, 
+                                                       current_parent_id, 
                                                        current_span_id))
                         current_span_id += 1
                     else:
@@ -453,21 +463,21 @@ def import_genes(bed_file, table_name, ref_id, file_id, additional_types, proper
 
                         # we're all UTR if we enter either of these
                         if (exon_hi <= thickStart and line[5] == '+') or (exon_lo >= thickEnd and line[5] == '-'):
-                            span.add_row(generate_gene_row(line,
-                                                           blockSizes[i],
-                                                           blockStarts[i],
-                                                           "5' UTR",
-                                                           default_row,
-                                                           current_parent_id,
+                            span.add_row(generate_gene_row(line, 
+                                                           blockSizes[i], 
+                                                           blockStarts[i], 
+                                                           "5' UTR", 
+                                                           default_row, 
+                                                           current_parent_id, 
                                                            current_span_id))
                             current_span_id += 1
                         elif (exon_hi <= thickStart and line[5] == '-') or (exon_lo >= thickEnd and line[5] == '+'):
-                            span.add_row(generate_gene_row(line,
-                                                           blockSizes[i],
-                                                           blockStarts[i],
-                                                           "3' UTR",
-                                                           default_row,
-                                                           current_parent_id,
+                            span.add_row(generate_gene_row(line, 
+                                                           blockSizes[i], 
+                                                           blockStarts[i], 
+                                                           "3' UTR", 
+                                                           default_row, 
+                                                           current_parent_id, 
                                                            current_span_id))
                             current_span_id += 1
 
@@ -475,12 +485,12 @@ def import_genes(bed_file, table_name, ref_id, file_id, additional_types, proper
                         elif (exon_lo < thickEnd and exon_hi > thickStart):
                             # entirely contained
                             if exon_lo >= thickStart and exon_hi <= thickEnd:
-                                span.add_row(generate_gene_row(line,
-                                                               blockSizes[i],
-                                                               blockStarts[i],
-                                                               "CDS",
-                                                               default_row,
-                                                               current_parent_id,
+                                span.add_row(generate_gene_row(line, 
+                                                               blockSizes[i], 
+                                                               blockStarts[i], 
+                                                               "CDS", 
+                                                               default_row, 
+                                                               current_parent_id, 
                                                                current_span_id))
                                 current_span_id += 1
                             else:
@@ -491,12 +501,12 @@ def import_genes(bed_file, table_name, ref_id, file_id, additional_types, proper
                                     else:
                                         UTR_type = "3' UTR"
                                     UTR_size = (min(blockSizes[i], thickStart - exon_lo))
-                                    span.add_row(generate_gene_row(line,
-                                                                   UTR_size,
-                                                                   blockStarts[i],
+                                    span.add_row(generate_gene_row(line, 
+                                                                   UTR_size, 
+                                                                   blockStarts[i], 
                                                                    UTR_type,
-                                                                   default_row,
-                                                                   current_parent_id,
+                                                                   default_row, 
+                                                                   current_parent_id, 
                                                                    current_span_id))
                                     current_span_id += 1
 
@@ -504,12 +514,12 @@ def import_genes(bed_file, table_name, ref_id, file_id, additional_types, proper
                                 CDS_size = blockSizes[i] - (max(exon_lo, thickStart) - exon_lo)
                                 CDS_size -= (exon_hi - min(exon_hi, thickEnd))
                                 CDS_start = (max(exon_lo, thickStart) - exon_lo) + blockStarts[i]
-                                span.add_row(generate_gene_row(line,
-                                                               CDS_size,
-                                                               CDS_start,
+                                span.add_row(generate_gene_row(line, 
+                                                               CDS_size, 
+                                                               CDS_start, 
                                                                "CDS",
-                                                               default_row,
-                                                               current_parent_id,
+                                                               default_row, 
+                                                               current_parent_id, 
                                                                current_span_id))
                                 current_span_id += 1
 
@@ -521,12 +531,12 @@ def import_genes(bed_file, table_name, ref_id, file_id, additional_types, proper
                                         UTR_type = "5' UTR"
                                     UTR_size = (min(blockSizes[i], exon_hi - thickEnd))
                                     UTR_start = blockStarts[i] + thickEnd - exon_lo
-                                    span.add_row(generate_gene_row(line,
-                                                                   UTR_size,
-                                                                   UTR_start,
+                                    span.add_row(generate_gene_row(line, 
+                                                                   UTR_size, 
+                                                                   UTR_start, 
                                                                    UTR_type,
-                                                                   default_row,
-                                                                   current_parent_id,
+                                                                   default_row, 
+                                                                   current_parent_id, 
                                                                    current_span_id))
                                     current_span_id += 1
 
@@ -574,16 +584,23 @@ def import_BED(**args):
         else:
             name = bed_filename+"_"+str(current_file)
         current_file += 1
-        bed_type = detect_type(import_filename)
+        bed_type = detect_type(import_filename)["type"]
+        delimiter = detect_type(import_filename)["delimiter"]
+
+        print >> sys.stderr, "Bed type is : " + bed_type
         if bed_type == "genes":
-            print >> sys.stderr,  "Importing as Genes Type"
-            job_outputs.append(import_genes(import_filename, name, reference, file_id, additional_types, property_keys, property_values, tags))
-        elif bed_type == "named_spans":
-            print >> sys.stderr,  "Importing as Spans Type (with names)"
-            job_outputs.append(import_named_spans(import_filename, name, reference, file_id, additional_types, property_keys, property_values, tags))
-        else:
+            print >> sys.stderr, "Importing as Genes Type"
+            job_outputs.append(import_genes(import_filename, name, reference, file_id, additional_types, property_keys, property_values, tags, delimiter))
+        elif bed_type == "spans" or bed_type == "bedDetail":
             print >> sys.stderr, "Importing as Spans Type"
-            job_outputs.append(import_spans(import_filename, name, reference, file_id, additional_types, property_keys, property_values, tags))
+            if bed_type == "bedDetail":
+                print >> sys.stderr, "input file is in 'bedDetails' format..."
+                bedDetail=True
+            else:
+                bedDetail=False
+            job_outputs.append(import_spans(import_filename, name, reference, file_id, additional_types, property_keys, property_values, tags, bedDetail, delimiter))
+        else:
+            raise dxpy.AppError("Unable to determine type of BED file")
 
         subprocess.check_call(" ".join(["rm", import_filename]), shell=True)
 
@@ -591,9 +608,75 @@ def import_BED(**args):
         subprocess.check_call(" ".join(["rm", bed_filename_uncomp]), shell=True)
 
     print json.dumps(job_outputs)
-
     return job_outputs
 
+def validate_line(line):
+    line_str = "\t".join(line)
+    entries = list(line)
+    
+    if len(entries) > 1:
+        try:
+            if int(entries[1]) < 0:
+                raise dxpy.AppError("The start position for one entry was unexpectedly negative. \nOffending line_str: " + line_str + "\nOffending value: " + str(entries[1]))
+        except ValueError:
+            raise dxpy.AppError("One of the start values could not be translated to an integer. " + "\nOffending line_str: " + line_str + "\nOffending value: " + str(entries[1]))
+    
+    if len(entries) > 2:    
+        try:
+            if int(entries[2]) < 0:
+                raise dxpy.AppError("The end position for one entry was unexpectedly negative. \nOffending line_str: " + line_str + "\nOffending value: " + str(entries[2]))
+        except ValueError:
+            raise dxpy.AppError("One of the end values could not be translated to an integer. " + "\nOffending line_str: " + line_str + "\nOffending value: " + str(entries[2]))
+        
+    if len(entries) > 4:    
+        try:
+            if entries[4] != "." and entries[4] != "-":
+                float(entries[4])
+        except ValueError:
+            raise dxpy.AppError("One of the score values for one entry could not be translated to a number. " + "\nOffending line_str: " + line_str + "\nOffending value: " + str(entries[4]))
+        
+    if len(entries) > 5:
+        if entries[5] != "+" and entries[5] != "-" and entries[5] != ".":
+            raise dxpy.AppError("The strand indicated for an element was not \"+\", \"-\", or \".\"" + "\nOffending line_str: " + line_str + "\nOffending value: " + str(entries[5]))
+    
+    if len(entries) > 6:
+        try:
+            if entries[6] != "." and entries[6] != "-":
+                if int(entries[6]) < 0:
+                    raise dxpy.AppError("The thickStart position for one entry was unexpectedly negative. \nOffending line_str: " + line_str + "\nOffending value: " + str(entries[6]))
+        except ValueError:
+            raise dxpy.AppError("One of the thickStart values could not be translated to an integer. " + "\nOffending line_str: " + line_str + "\nOffending value: " + str(entries[6]))
+    
+    if len(entries) > 7:    
+        try:
+            if entries[7] != "." and entries[7] != "-":
+                if int(entries[7]) < 0:
+                    raise dxpy.AppError("The thickEnd position for one entry was unexpectedly negative. \nOffending line_str: " + line_str + "\nOffending value: " + str(entries[7]))
+        except ValueError:
+            raise dxpy.AppError("One of the thickEnd values could not be translated to an integer. " + "\nOffending line_str: " + line_str + "\nOffending value: " + str(entries[7]))
+    
+    if len(entries) > 9:
+        try:
+            if int(entries[9]) < 0:
+                raise dxpy.AppError("The number of exons (blockCount) for one entry was unexpectedly negative. \nOffending line_str: " + line_str + "\nOffending value: " + str(entries[9]))
+        except ValueError:
+            raise dxpy.AppError("One of the thickEnd values could not be translated to an integer. " + "\nOffending line_str: " + line_str + "\nOffending value: " + str(entries[9]))
+    
+    if len(entries) > 10:    
+        try:
+            entries[10] = entries[10].rstrip(",").split(",")
+            blockStarts = [int(entries[10][n]) for n in range(int(entries[9]))]
+        except:
+            raise dxpy.AppError("Could not parse the blockSizes entry as a comma-separated list of integers \nOffending line_str: " + line_str + "\nOffending value: " + str(entries[10]))
+        
+    if len(entries) > 11:
+        try:
+            entries[11] = entries[11].rstrip(",").split(",")
+            blockStarts = [int(entries[11][n]) for n in range(int(entries[9]))]
+        except:
+            raise dxpy.AppError("Could not parse the blockStarts entry as a comma-separated list of integers \nOffending line_str: " + line_str + "\nOffending value: " + str(entries[11]))
+
+
 def main(**args):
     import_BED(**args)
 
