@@ -33,6 +33,8 @@ from dxpy import logger
 
 from dxpy.utils.resolver import resolve_path, is_container_id
 from dxpy.app_categories import APP_CATEGORIES
+from dxpy.exceptions import err_exit
+from dxpy.utils.printing import BOLD
 
 parser = argparse.ArgumentParser(description="Uploads a DNAnexus App.")
 
@@ -42,7 +44,7 @@ APP_VERSION_RE = re.compile("^([1-9][0-9]*|0)\.([1-9][0-9]*|0)\.([1-9][0-9]*|0)(
 parser.add_argument("src_dir", help="App or applet source directory (default: current directory)", nargs='?')
 
 parser.set_defaults(mode="app")
-parser.add_argument("--create-app", help="Create an app (otherwise, creates an applet)", action="store_const",
+parser.add_argument("--app", "--create-app", help="Create an app (otherwise, creates an applet)", action="store_const",
                     dest="mode", const="app")
 parser.add_argument("--create-applet", help=argparse.SUPPRESS, action="store_const", dest="mode", const="applet")
 parser.add_argument("-d", "--destination", help="Specifies the destination project, destination folder, and/or name for the applet, in the form [PROJECT_NAME_OR_ID:][/[FOLDER/][NAME]]. Overrides the project, folder, and name fields of the dxapp.json, if they were supplied.", default='.')
@@ -135,6 +137,8 @@ parser.set_defaults(dry_run=False)
 parser.add_argument("--dry-run", "-n", help="Do not create an app(let): only perform local checks and compilation steps, and show the spec of the app(let) that would have been created.", action="store_true", dest="dry_run")
 parser.add_argument("--no-dry-run", help=argparse.SUPPRESS, action="store_false", dest="dry_run")
 
+parser.add_argument("--extra-args", help="Arguments (in JSON format) to pass to the /applet/new API method, overriding all other settings")
+parser.add_argument("--run", help="Run the app or applet after building it (options following this are passed to "+BOLD("dx run")+")", nargs=argparse.REMAINDER)
 
 def _get_timestamp_version_suffix(version):
     if "+" in version:
@@ -498,7 +502,7 @@ def _build_app_remote(mode, src_dir, publish=False, destination_override=None,
                     if sys.stderr.isatty():
                         if last_console_update > 0:
                             sys.stderr.write("\r")
-                        sys.stderr.write("Compressing target directory %s... (%s kb)" % (src_dir, locale.format("%d", (total_num_bytes / 1024,), grouping=True),))
+                        sys.stderr.write("Compressing target directory {dir}... ({kb_so_far:,} kb)".format(dir=src_dir, kb_so_far=total_num_bytes / 1024))
                         sys.stderr.flush()
                         last_console_update = current_time
                     elif not printed_static_message:
@@ -550,7 +554,8 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publ
                              version_override=None, bill_to_override=None, use_temp_build_project=True,
                              do_parallel_build=True, do_version_autonumbering=True, do_try_update=True,
                              dx_toolkit_autodep="stable", do_build_step=True, do_upload_step=True, do_check_syntax=True,
-                             dry_run=False, return_object_dump=False):
+                             dry_run=False, return_object_dump=False,
+                             **kwargs):
     app_json = _parse_app_spec(src_dir)
 
     _verify_app_source_dir(src_dir, enforce=do_check_syntax)
@@ -599,7 +604,8 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publ
                 override_folder=override_folder,
                 override_name=override_applet_name,
                 dx_toolkit_autodep=dx_toolkit_autodep,
-                dry_run=dry_run)
+                dry_run=dry_run,
+                **kwargs)
         except:
             # Avoid leaking any bundled_resources files we may have
             # created, if applet creation fails. Note that if
@@ -645,10 +651,10 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publ
                 print >> sys.stderr, "You can publish this app with:"
                 print >> sys.stderr, "  dx api app-%s/%s publish \"{\\\"makeDefault\\\": true}\"" % (app_describe["name"], app_describe["version"])
 
-            return app_describe if return_object_dump else None
+            return app_describe if return_object_dump else {"id": app_id}
 
         elif mode == "applet":
-            return dxpy.api.applet_describe(applet_id) if return_object_dump else None
+            return dxpy.api.applet_describe(applet_id) if return_object_dump else {"id": applet_id}
         else:
             raise dxpy.app_builder.AppBuilderException("Unrecognized mode %r" % (mode,))
 
@@ -695,6 +701,8 @@ def main(**kwargs):
     if args.overwrite and args.archive:
         parser.error("Options -f/--overwrite and -a/--archive cannot be specified together")
 
+    extra_args = json.loads(args.extra_args) if args.extra_args else {}
+
     if not args.remote:
         # LOCAL BUILD
 
@@ -717,7 +725,8 @@ def main(**kwargs):
                 do_upload_step=args.upload_step,
                 do_check_syntax=args.check_syntax,
                 dry_run=args.dry_run,
-                return_object_dump=args.json
+                return_object_dump=args.json,
+                **extra_args
                 )
 
             if output is not None:
@@ -728,6 +737,12 @@ def main(**kwargs):
             print >> sys.stderr, "Error: %s" % (e.message,)
             sys.exit(3)
 
+        if args.run is not None:
+            if output is None:
+                err_exit("The --run option was given, but no executable was created")
+            cmd = os.path.join(os.environ['DNANEXUS_HOME'], 'bin', 'dx')
+            os.execv(cmd, ['dx', 'run', output['id']] + args.run)
+
         return
 
     else:
