@@ -24,6 +24,7 @@ import pexpect
 import dxpy
 from dxpy.scripts import dx_build_app
 from dxpy_testutil import DXTestCase
+import dxpy_testutil as testutil
 
 @contextmanager
 def chdir(dirname=None):
@@ -88,7 +89,7 @@ def makeGenomeObject():
     # would be huge, for hg19). Importers and exporters that need to
     # look at the full sequence file can't be run on this test
     # contigset.
-    sequence_file = dxpy.upload_string("")
+    sequence_file = dxpy.upload_string("", hidden=True)
 
     genome_record = dxpy.new_dxrecord()
     genome_record.set_details({
@@ -188,10 +189,113 @@ class TestDXClient(DXTestCase):
         run(u"dx find jobs --project :")
         run(u"dx find data --project :")
 
+    def test_dx_object_tagging(self):
+        the_tags = [u"Σ1=n", u"helloo0", u"ωω"]
+        # tag
+        record_id = run(u"dx new record Ψ --brief").strip()
+        run(u"dx tag Ψ " + u" ".join(the_tags))
+        mytags = dxpy.describe(record_id)['tags']
+        for tag in the_tags:
+            self.assertIn(tag, mytags)
+        # untag
+        run(u"dx untag Ψ " + u" ".join(the_tags[:2]))
+        mytags = dxpy.describe(record_id)['tags']
+        for tag in the_tags[:2]:
+            self.assertNotIn(tag, mytags)
+        self.assertIn(the_tags[2], mytags)
+
+        # -a flag
+        second_record_id = run(u"dx new record Ψ --brief").strip()
+        self.assertNotEqual(record_id, second_record_id)
+        run(u"dx tag -a Ψ " + u" ".join(the_tags))
+        mytags = dxpy.describe(record_id)['tags']
+        for tag in the_tags:
+            self.assertIn(tag, mytags)
+        second_tags = dxpy.describe(second_record_id)['tags']
+        for tag in the_tags:
+            self.assertIn(tag, second_tags)
+
+        run(u"dx untag -a Ψ " + u" ".join(the_tags))
+        mytags = dxpy.describe(record_id)['tags']
+        self.assertEqual(len(mytags), 0)
+        second_tags = dxpy.describe(second_record_id)['tags']
+        self.assertEqual(len(second_tags), 0)
+
+    def test_dx_project_tagging(self):
+        the_tags = [u"$my.tag", u"secoиdtag", u"тhird тagggg"]
+        # tag
+        run(u"dx tag : \\" + the_tags[0] + u" " + the_tags[1] + u" '" + the_tags[2] + u"'")
+        mytags = dxpy.describe(self.project)['tags']
+        for tag in the_tags:
+            self.assertIn(tag, mytags)
+        # untag
+        run(u"dx untag : \\" + the_tags[0] + u" '" + the_tags[2] + u"'")
+        mytags = dxpy.describe(self.project)['tags']
+        self.assertIn(the_tags[1], mytags)
+        for tag in [the_tags[0], the_tags[2]]:
+            self.assertNotIn(tag, mytags)
+
+    def test_dx_object_properties(self):
+        property_names = [u"Σ_1^n", u"helloo0", u"ωω"]
+        property_values = [u"n", u"world z", u"ω()"]
+        # set_properties
+        record_id = run(u"dx new record Ψ --brief").strip()
+        run(u"dx set_properties Ψ " + u" ".join([u"'" + prop[0] + u"'='" + prop[1] + u"'" for prop in zip(property_names, property_values)]))
+        my_properties = dxpy.api.record_describe(record_id, {"properties": True})['properties']
+        for (name, value) in zip(property_names, property_values):
+            self.assertIn(name, my_properties)
+            self.assertEqual(value, my_properties[name])
+        # unset_properties
+        run(u"dx unset_properties Ψ '" + u"' '".join(property_names[:2]) + u"'")
+        my_properties = dxpy.api.record_describe(record_id, {"properties": True})['properties']
+        for name in property_names[:2]:
+            self.assertNotIn(name, my_properties)
+        self.assertIn(property_names[2], my_properties)
+        self.assertEqual(property_values[2], my_properties[property_names[2]])
+
+        # -a flag
+        second_record_id = run(u"dx new record Ψ --brief").strip()
+        self.assertNotEqual(record_id, second_record_id)
+        run(u"dx set_properties -a Ψ " + u" ".join([u"'" + prop[0] + u"'='" + prop[1] + u"'" for prop in zip(property_names, property_values)]))
+        my_properties = dxpy.api.record_describe(record_id, {"properties": True})['properties']
+        for (name, value) in zip(property_names, property_values):
+            self.assertIn(name, my_properties)
+            self.assertEqual(value, my_properties[name])
+        second_properties = dxpy.api.record_describe(second_record_id, {"properties": True})['properties']
+        for (name, value) in zip(property_names, property_values):
+            self.assertIn(name, my_properties)
+            self.assertEqual(value, my_properties[name])
+
+        run(u"dx unset_properties -a Ψ '" + u"' '".join(property_names) + u"'")
+        my_properties = dxpy.api.record_describe(record_id, {"properties": True})['properties']
+        self.assertEqual(len(my_properties), 0)
+        second_properties = dxpy.api.record_describe(second_record_id, {"properties": True})['properties']
+        self.assertEqual(len(second_properties), 0)
+
+    def test_dx_project_properties(self):
+        property_names = [u"$my.prop", u"secoиdprop", u"тhird prop"]
+        property_values = [u"$hello.world", u"Σ2,n", u"stuff"]
+        # set_properties
+        run(u"dx set_properties : " + u" ".join([u"'" + prop[0] + u"'='" + prop[1] + u"'" for prop in zip(property_names, property_values)]))
+        my_properties = dxpy.api.project_describe(self.project, {"properties": True})['properties']
+        for (name, value) in zip(property_names, property_values):
+            self.assertIn(name, my_properties)
+            self.assertEqual(value, my_properties[name])
+        # unset_properties
+        run(u"dx unset_properties : '" + property_names[0] + u"' '" + property_names[2] + u"'")
+        my_properties = dxpy.api.project_describe(self.project, {"properties": True})['properties']
+        self.assertIn(property_names[1], my_properties)
+        self.assertEqual(property_values[1], my_properties[property_names[1]])
+        for name in [property_names[0], property_names[2]]:
+            self.assertNotIn(name, my_properties)
+
     def test_dx_describe_project(self):
         describe_output = run(u"dx describe :").strip()
         self.assertTrue(re.search(r'ID\s+%s.*\n.*\nName\s+dxclient_test_pr\xc3\xb6ject' % (self.project,),
                                   describe_output))
+        self.assertNotIn('Properties', describe_output)
+        describe_output = run(u"dx describe : --verbose")
+        self.assertIn('Properties', describe_output)
 
     def test_dx_remove_project_by_name(self):
         # TODO: this test makes no use of the DXTestCase-provided
@@ -333,6 +437,28 @@ class TestDXClient(DXTestCase):
         shell1.sendline("bash -c 'dx env'")
         expect_dx_env_cwd(shell1, "sessiontest1")
 
+    @unittest.skipUnless(testutil.TEST_RUN_JOBS,
+                         'skipping test that would run jobs')
+    def test_dx_run_extra_args(self):
+        # success
+        applet_id = dxpy.api.applet_new({"project": self.project,
+                                         "dxapi": "1.0.0",
+                                         "runSpec": {"interpreter": "bash",
+                                                     "code": "echo 'hello'"}
+                                         })['id']
+        job_id = run("dx run " + applet_id + ' -inumber=32 --name overwritten_name --delay-workspace-destruction --extra-args \'{"input": {"second": true}, "name": "new_name"}\' --brief -y').strip()
+        job_desc = dxpy.api.job_describe(job_id)
+        self.assertTrue(job_desc['delayWorkspaceDestruction'])
+        self.assertEqual(job_desc['name'], 'new_name')
+        self.assertIn('number', job_desc['input'])
+        self.assertEqual(job_desc['input']['number'], 32)
+        self.assertIn('second', job_desc['input'])
+        self.assertEqual(job_desc['input']['second'], True)
+
+        # parsing error
+        with self.assertSubprocessFailure(stderr_regexp='JSON', exit_code=3):
+            run("dx run " + applet_id + " --extra-args not-a-JSON-string")
+
 class TestDXBuildApp(DXTestCase):
     def setUp(self):
         self.temp_file_path = tempfile.mkdtemp()
@@ -406,8 +532,8 @@ class TestDXBuildApp(DXTestCase):
         with self.assertSubprocessFailure(stderr_regexp='Could not parse dxapp\.json file', exit_code=3):
             run("dx build " + app_dir)
 
-    @unittest.skipIf('DXTEST_FULL' not in os.environ,
-                     'skipping test that would create apps')
+    @unittest.skipUnless(testutil.TEST_CREATE_APPS,
+                         'skipping test that would create apps')
     def test_build_app(self):
         app_spec = {
             "name": "minimal_app",
@@ -426,8 +552,8 @@ class TestDXBuildApp(DXTestCase):
         self.assertEqual(app_describe["name"], "minimal_app")
         self.assertFalse("published" in app_describe)
 
-    @unittest.skipIf('DXTEST_FULL' not in os.environ,
-                     'skipping test that would create apps')
+    @unittest.skipUnless(testutil.TEST_CREATE_APPS,
+                         'skipping test that would create apps')
     def test_invalid_project_context(self):
         app_spec = {
             "name": "invalid_project_context",
@@ -486,8 +612,8 @@ class TestDXBuildApp(DXTestCase):
         with self.assertSubprocessFailure(exit_code=3):
             run("dx describe " + applet_id)
 
-    @unittest.skipIf('DXTEST_FULL' not in os.environ,
-                     'skipping test that would create apps')
+    @unittest.skipUnless(testutil.TEST_CREATE_APPS,
+                         'skipping test that would create apps')
     def test_update_app_categories(self):
         app1_spec = {
             "name": "update_app_categories",
@@ -515,8 +641,8 @@ class TestDXBuildApp(DXTestCase):
         run("dx build --create-app --json " + app_dir)
         self.assertEquals(json.loads(run("dx api " + app_id + " listCategories"))["categories"], ['B'])
 
-    @unittest.skipIf('DXTEST_FULL' not in os.environ,
-                     'skipping test that would create apps')
+    @unittest.skipUnless(testutil.TEST_CREATE_APPS,
+                         'skipping test that would create apps')
     def test_build_app_autonumbering(self):
         app_spec = {
             "name": "build_app_autonumbering",
@@ -821,5 +947,5 @@ FOO.12345678\t0\t1\t54932369\t60\t7M1D93M\t*\t0\t0\tTAATAAGGTTGTTGTTGTTGTT\t1:1A
 
 if __name__ == '__main__':
     if 'DXTEST_FULL' not in os.environ:
-        sys.stderr.write('WARNING: env var DXTEST_FULL is not set; tests that create apps will not be run\n')
+        sys.stderr.write('WARNING: env var DXTEST_FULL is not set; tests that create apps or run jobs will not be run\n')
     unittest.main()
